/***************************************************************************
                     configdlg.cpp  -  The settings dialog.
                     -------------------
begin                : Do Okt 16 14:35:18 CEST 2003
copyright            : (C) 2003 by Mark Kretschmann
email                :
***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "configdlg.h"
//#include "browserwidget.h"
#include "browserwin.h" //FIXME emit signal/postEvent instead of call function!
#include "Options1.h"
#include "Options2.h"
#include "Options3.h"
#include "Options4.h"
#include "playerapp.h"
//#include "playlistwidget.h"

#include <qcheckbox.h>
#include <qcolor.h>
#include <qcombobox.h>
#include <qbuttongroup.h>
#include <qfont.h>
#include <qlabel.h>
#include <qpushbutton.h>
#include <qradiobutton.h>
#include <qspinbox.h>
#include <qvbox.h>
#include <qsignalmapper.h>

#include <kcolorbutton.h>
#include <kdialogbase.h>
#include <kfontdialog.h>
#include <kiconloader.h>
#include <klocale.h>


ConfigDlg::ConfigDlg() : KDialogBase( KDialogBase::IconList, i18n( "Options" ),
                                              KDialogBase::Ok | KDialogBase::Apply | KDialogBase::Cancel,
                                              KDialogBase::Ok, 0, 0, false )
{
    KIconLoader iconLoader;

    // PAGE: OPTIONS1 (misc) ===========================================
    QVBox *pVBox1 = addVBoxPage( i18n( "General" ) , i18n( "Configure general options" ),
                                 iconLoader.loadIcon( "misc", KIcon::NoGroup, KIcon::SizeMedium ) );

    m_pOptions1 = new Options1( pVBox1 );
    initOptions1();

    // PAGE: OPTIONS2 (fonts) ===========================================
    QVBox *pVBox2 = addVBoxPage( i18n( "Fonts" ) , i18n( "Configure fonts" ),
                                 iconLoader.loadIcon( "fonts", KIcon::NoGroup, KIcon::SizeMedium ) );

    m_pOptions2 = new Options2( pVBox2 );
    initOptions2();

    // PAGE: OPTIONS3 (colors) ==========================================
    QVBox *pVBox3 = addVBoxPage( i18n( "Colors" ) , i18n( "Configure Colors" ),
                                 iconLoader.loadIcon( "colors", KIcon::NoGroup, KIcon::SizeMedium ) );

    m_pOptions3 = new Options3( pVBox3 );
    initOptions3();
    
    // PAGE: OPTIONS4 (playback) ==========================================
    QVBox *pVBox4 = addVBoxPage( i18n( "Playback" ), i18n( "Configure playback" ),
                                 iconLoader.loadIcon( "kmix", KIcon::NoGroup, KIcon::SizeMedium ) );
    
    m_pOptions4 = new Options4( pVBox4 );
    initOptions4();

    enableButtonApply( false );
    setInitialSize( QSize( 480, 430 ) );
}


ConfigDlg::~ConfigDlg()
{
}


// METHODS ======================================================================

void ConfigDlg::slotOk()
{
    slotSave();
    accept();
}


void ConfigDlg::slotApply()
{
    slotSave();
    enableButtonApply( false );
}


void ConfigDlg::slotCancel()
{
    reject();
}


void ConfigDlg::slotSave()
{
    saveOptions1();
    saveOptions2();
    saveOptions3();
    saveOptions4();
}


void ConfigDlg::slotChanged()
{
    enableButtonApply( true );
}


// INIT ========================================================================

void ConfigDlg::initOptions1()
{
    m_pOptions1->checkBox1->setChecked( pApp->m_optSavePlaylist );
    m_pOptions1->checkBox7->setChecked( pApp->m_optConfirmClear );
    m_pOptions1->checkBox6->setChecked( pApp->m_optConfirmExit );
    m_pOptions1->checkBox4->setChecked( pApp->m_optReadMetaInfo );
    m_pOptions1->checkBox3->setChecked( pApp->m_optShowTrayIcon );
    m_pOptions1->checkBox5->setChecked( pApp->m_optHidePlaylistWindow );

    if ( pApp->m_optDropMode == "Ask" )
        m_pOptions1->comboBox1->setCurrentItem( 0 );
    if ( pApp->m_optDropMode == "Recursively" )
        m_pOptions1->comboBox1->setCurrentItem( 1 );
    if ( pApp->m_optDropMode == "NonRecursively" )
        m_pOptions1->comboBox1->setCurrentItem( 2 );

    connect( m_pOptions1->checkBox1, SIGNAL( stateChanged( int ) ),
             this, SLOT( slotChanged() ) );
    connect( m_pOptions1->checkBox7, SIGNAL( stateChanged( int ) ),
             this, SLOT( slotChanged() ) );
    connect( m_pOptions1->checkBox6, SIGNAL( stateChanged( int ) ),
             this, SLOT( slotChanged() ) );
    connect( m_pOptions1->checkBox4, SIGNAL( stateChanged( int ) ),
             this, SLOT( slotChanged() ) );
    connect( m_pOptions1->checkBox3, SIGNAL( stateChanged( int ) ),
             this, SLOT( slotChanged() ) );
    connect( m_pOptions1->checkBox5, SIGNAL( stateChanged( int ) ),
             this, SLOT( slotChanged() ) );
    connect( m_pOptions1->comboBox1, SIGNAL( activated( int ) ),
             this, SLOT( slotChanged() ) );
}


void ConfigDlg::initOptions2()
{
    //m_pOptions2->chkBoxUseCustomFonts->setChecked( pApp->m_optUseCustomFonts );
    m_pOptions2->previewPW ->setFont( pApp->m_optPlayerWidgetFont );
    m_pOptions2->previewPWS->setFont( pApp->m_optPlayerWidgetScrollFont );
    m_pOptions2->previewBW ->setFont( pApp->m_optBrowserWindowFont );

    QSignalMapper *mapper = new QSignalMapper( this );
    mapper->setMapping( m_pOptions2->choosePW,  int(ConfigDlg::PW) );
    mapper->setMapping( m_pOptions2->choosePWS, int(ConfigDlg::PWS) );
    mapper->setMapping( m_pOptions2->chooseBW,  int(ConfigDlg::BW) );

    connect( mapper, SIGNAL( mapped( int ) ),
             this, SLOT( slotChooseFont( int ) ) );
    connect( m_pOptions2->choosePW, SIGNAL( clicked() ),
             mapper, SLOT( map() ) );
    connect( m_pOptions2->choosePWS, SIGNAL( clicked() ),
             mapper, SLOT( map() ) );
    connect( m_pOptions2->chooseBW, SIGNAL( clicked() ),
             mapper, SLOT( map() ) );
    connect( mapper, SIGNAL( mapped( int ) ),
             this, SLOT( slotChanged() ) );
    connect( m_pOptions2->chkBoxUseCustomFonts, SIGNAL( toggled( bool ) ),
             this, SLOT( slotChanged() ) );
}


void ConfigDlg::initOptions3()
{
    m_pOptions3->chkBoxTransparency->setDisabled( true );

    //m_pOptions3->chkBoxCustomColor->setChecked( pApp->m_optBrowserUseCustomColors );
    m_pOptions3->kColorButtonBg->setColor( pApp->m_optBrowserBgColor );
    m_pOptions3->kColorButtonFg->setColor( pApp->m_optBrowserFgColor );

    connect( m_pOptions3->chkBoxTransparency, SIGNAL( stateChanged( int ) ),
             this, SLOT( slotChanged() ) );
    connect( m_pOptions3->chkBoxCustomColor, SIGNAL( toggled( bool ) ),
             this, SLOT( slotChanged() ) ); //is a groupBox
    connect( m_pOptions3->kColorButtonBg, SIGNAL( changed( const QColor& ) ),
             this, SLOT( slotChooseBrowserWinBgColor( const QColor& ) ) );
    connect( m_pOptions3->kColorButtonFg, SIGNAL( changed( const QColor& ) ),
             this, SLOT( slotChooseBrowserWinFgColor( const QColor& ) ) );
}


void ConfigDlg::initOptions4()
{
    //automatically enable/disable the right stuff
    connect( m_pOptions4->radioButtonCrossfadePlayback, SIGNAL( toggled( bool ) ),
             m_pOptions4->crossfadeLength, SLOT( setEnabled( bool ) ) );
    connect( m_pOptions4->radioButtonCrossfadePlayback, SIGNAL( toggled( bool ) ),
             m_pOptions4->crossfadeLengthLabel, SLOT( setEnabled( bool ) ) );
    connect( m_pOptions4->radioButtonNormalPlayback, SIGNAL( toggled( bool ) ),
             m_pOptions4->trackDelayLength, SLOT( setEnabled( bool ) ) );
    connect( m_pOptions4->radioButtonNormalPlayback, SIGNAL( toggled( bool ) ),
             m_pOptions4->trackDelayLengthLabel, SLOT( setEnabled( bool ) ) );

    //FIXME: remove Qt=3.2 from ui files

    if ( pApp->m_optXFade ) //the radiobuttons are exclusive
       m_pOptions4->radioButtonCrossfadePlayback->setChecked( true );
    else
       m_pOptions4->radioButtonNormalPlayback->setChecked( true );

    m_pOptions4->crossfadeLength->setValue( pApp->m_optXFadeLength );
    m_pOptions4->trackDelayLength->setValue( pApp->m_optTrackDelay );
    m_pOptions4->checkBoxResumePlayback->setChecked( pApp->m_optResumePlayback );
    m_pOptions4->checkBoxSwMixerOnly->setChecked( pApp->m_optSoftwareMixerOnly );
    m_pOptions4->checkBoxTitleStream->setChecked( pApp->m_optTitleStream );

    //BEWARE! if there are more than 2 playback types, you must connect all the radiobuttons in the group to stateChanged()
    connect( m_pOptions4->radioButtonCrossfadePlayback, SIGNAL( toggled( bool ) ),
             this, SLOT( slotChanged() ) );
    connect( m_pOptions4->radioButtonNormalPlayback, SIGNAL( toggled( bool ) ),
             this, SLOT( slotChanged() ) );
    connect( m_pOptions4->checkBoxResumePlayback, SIGNAL( stateChanged( int ) ),
             this, SLOT( slotChanged() ) );
    connect( m_pOptions4->checkBoxSwMixerOnly, SIGNAL( stateChanged( int ) ),
             this, SLOT( slotChanged() ) );
    connect( m_pOptions4->checkBoxTitleStream, SIGNAL( stateChanged( int ) ),
             this, SLOT( slotChanged() ) );
    connect( m_pOptions4->trackDelayLength, SIGNAL( valueChanged( int ) ),
             this, SLOT( slotChanged() ) );
    connect( m_pOptions4->crossfadeLength, SIGNAL( valueChanged( int ) ),
             this, SLOT( slotChanged() ) );
}

// SAVE ========================================================================

void ConfigDlg::saveOptions1()
{
    pApp->m_optSavePlaylist = m_pOptions1->checkBox1->isChecked();
    pApp->m_optConfirmClear = m_pOptions1->checkBox7->isChecked();
    pApp->m_optConfirmExit = m_pOptions1->checkBox6->isChecked();
    pApp->m_optReadMetaInfo = m_pOptions1->checkBox4->isChecked();
    pApp->m_optShowTrayIcon = m_pOptions1->checkBox3->isChecked();
    pApp->m_optHidePlaylistWindow = m_pOptions1->checkBox5->isChecked();

    emit sigShowTrayIcon( pApp->m_optShowTrayIcon );

    if( m_pOptions1->comboBox1->currentItem() == 0 )
            pApp->m_optDropMode = "Ask";
    if( m_pOptions1->comboBox1->currentItem() == 1 )
            pApp->m_optDropMode = "Recursively";
    if( m_pOptions1->comboBox1->currentItem() == 2 )
            pApp->m_optDropMode = "NonRecursively";
}


void ConfigDlg::saveOptions2()
{
    //FIXME when we use Qt 3.2, re-enable this and add a checkbox to groupbox
    pApp->m_optUseCustomFonts = true; //m_pOptions2->chkBoxUseCustomFonts->isChecked();
    pApp->m_optPlayerWidgetFont = m_pOptions2->previewPW->font();
    pApp->m_optPlayerWidgetScrollFont = m_pOptions2->previewPWS->font();
    pApp->m_optBrowserWindowFont = m_pOptions2->previewBW->font();

    pApp->m_pBrowserWin->slotUpdateFonts();
}


void ConfigDlg::saveOptions3()
{
    //FIXME remember custom colours when user chooses to use defaults rather than set custom colours to defaults

    QColor bg, fg;

    //FIXME when we use Qt 3.2, re-enable this and add a checkbox to groupbox
    if( pApp->m_optBrowserUseCustomColors = true ) //m_pOptions3->chkBoxCustomColor->isChecked() )
    {
       fg = m_pOptions3->kColorButtonFg->color();
       bg = m_pOptions3->kColorButtonBg->color();

    } else {

       fg = pApp->m_fgColor;
       bg = pApp->m_bgColor;
    }

    pApp->m_optBrowserFgColor = fg;
    pApp->m_optBrowserBgColor = bg;

    //FIXME doesn't update KHIstoryCombo for some reason

    pApp->setupColors();
}


void ConfigDlg::saveOptions4()
{
    //FIXME this is a hacky way to do this
    //      when we have a better options/config system then rely on seperate setting to specify playback type
    //      for now this should work, although if built on, eventually bugs will emerge (so don't extend!)

    pApp->m_optXFadeLength = m_pOptions4->crossfadeLength->value();
    pApp->m_optXFade = m_pOptions4->radioButtonCrossfadePlayback->isChecked();
    
    if ( m_pOptions4->radioButtonCrossfadePlayback->isChecked() )
       pApp->m_optTrackDelay = 0;
    else
       pApp->m_optTrackDelay = m_pOptions4->trackDelayLength->value();

    pApp->m_optResumePlayback = m_pOptions4->checkBoxResumePlayback->isChecked();
    pApp->m_optSoftwareMixerOnly = m_pOptions4->checkBoxSwMixerOnly->isChecked();
    pApp->m_optTitleStream = m_pOptions4->checkBoxTitleStream->isChecked();
}


// SLOTS ========================================================================

void ConfigDlg::slotChooseFont( int i )
{
    QWidget *widget;

    switch( static_cast<FontEnum>(i) )
    {
    case ConfigDlg::PW:
       widget = m_pOptions2->previewPW;
       break;
    case ConfigDlg::PWS:
       widget = m_pOptions2->previewPWS;
       break;
    case ConfigDlg::BW:
    default:
       widget = m_pOptions2->previewBW;
    }

    QFont font( widget->font() );

    if ( KFontDialog::getFont( font ) == QDialog::Accepted )
    {
       widget->setFont( font );
       slotChanged();
    }
}


void ConfigDlg::slotChooseBrowserWinBgColor( const QColor & col )
{
    m_pOptions3->kColorButtonBg->setColor( col );

    slotChanged();
}


void ConfigDlg::slotChooseBrowserWinFgColor( const QColor & col )
{
    m_pOptions3->kColorButtonFg->setColor( col );

    slotChanged();
}

#include "configdlg.moc"
